import pycanlii.pycanliibase as base
import pycanlii.enums as enums
import requests

class LegislationDatabase(base.PyCanliiBase):
    """
    A database of CanLII Legislation. This object is both indexable and iterable.

    Attributes:
        :name: A string representing the name of this LegislationDatabase
        :type: An instance of the LegislationType enum indicating what kind of Legislation this LegislationDatabasecontains
        :id: A string representing the databaseId of this LegislationDatabase
        :jurisdiction: An instance of a Jurisdiction enum representing the Jurisdiction of the Legislation in this LegislationDatabase
    """

    def __init__(self, data, apikey, language=enums.Language.en):
        base.PyCanliiBase.__init__(self, apikey, language)
        self.name = data['name']

        if (data['type'] == "REGULATION"):
            self.type = enums.LegislationType.Regulation
        elif (data['type'] == "STATUTE"):
            self.type = enums.LegislationType.Statute
        else:
            # cause of the API this should never actually happen
            raise Exception("Invalid legislation type, this should never happen, please open an issue on our " +
                            "git repo at https://github.com/sherlocke/pycanlii and include your stack trace " +
                            "or alternatively contribute and submit a pull request :)")

        self.id = data["databaseId"]
        # still need to add jurisdiction although for basic functionality, strictly speaking, not required
        self.jurisdiction = enums.Jurisdiction[data['jurisdiction']]
        self._legislation = []
        self._populated = False

    def _getLegislation(self):
        legis = self._request("http://api.canlii.org/v1/legislationBrowse",
                              True, self.id).json()['legislations']
        for legislation in legis:
            self._legislation.append(Legislation(legislation, self._key, self._lang))

    def __iter__(self):
        if not self._populated:
            self._getLegislation()
            self._populated = True
        return self._legislation.__iter__()

    def __getitem__(self, item):
        if not self._populated:
            self._getLegislation()
            self._populated = True
        return self._legislation[item]


class Legislation(base.PyCanliiBase):
    """
    An object representing Legislation on CanLII

    Attributes:
        :databaseId: A string representing the databaseId of this legislation
        :legislationId: A string representing the legislationId of this legislation
        :title: A string representing the title of this legislation
        :content: A BeautifulSoup object representing the HTML content of this legislation
        :url: A string representing the URL where this legislation can be found
        :dateScheme: An instance of the DateScheme enum representing the dateScheme of this legislation
        :startDate: A date object representing the start date of this legislation
        :endDate: A date object representing the end date of this legislation. If it has no end date yet, it will be none
        :repealed: A boolean representing whether or not this legislation has been repealed or not
    """

    def __init__(self, data, apikey, language=enums.Language.en):
        base.PyCanliiBase.__init__(self, apikey, language)
        self.databaseId = data['databaseId']
        self.legislationId = data['legislationId']
        self.title = data['title']
        self._citation = data['citation']

        self._populated = False
        self._url = None
        self._dateScheme = None
        self._startDate = None
        self._endDate = None
        self._repealed = None

        # Used to store the content of the Legislation
        self._content = None

        if (data['type'] == "REGULATION"):
            self.type = enums.LegislationType.Regulation
        elif (data['type'] == "STATUTE"):
            self.type = enums.LegislationType.Statute
        else:
            # cause of the API this should never actually happen
            raise Exception("Invalid legislation type")


    def _populate(self):
        if not self._populated:
            legis = self._request("http://api.canlii.org/v1/legislationBrowse",
                                  True, self.databaseId, self.legislationId)
            legis = legis.json()
            self._url = legis['url']
            self._dateScheme = enums.DateScheme[legis['dateScheme']]
            self._startDate = self._getDate(legis['startDate'])
            self._endDate = self._getDate(legis['endDate'])
        
            if (legis['repealed']  == 'NO'):
                self._repealed = False
            else:
                self._repealed = True

            self._populated = True

    @property
    def content(self):
        if not self._content:
            req = requests.get(self.url)
            self._content = req.content

        return self._content

    @property
    def url(self):
        self._populate()
        return self._url

    @property
    def citation(self):
        return self._citation

    @property
    def dateScheme(self):
        self._populate()
        return self._dateScheme

    @property
    def startDate(self):
        self._populate()
        return self._startDate

    @property
    def endDate(self):
        self._populate()
        return self._endDate

    @property
    def repealed(self):
        self._populate()
        return self._repealed





import requests
import datetime

class PyCanliiBase(object):
    """
    The base object in the pycanlii library. All objects in the library inherit from it.
    """

    def __init__(self, apikey, language):
        self._key = apikey
        self._lang = language

    def _request(self, url, authenticated, *url_variables, **query_parameters):
        """
        Sends a request to the input url, with the url parameters
        place in order with a / between each and with the
        query parameters input.

        :return: A response object
        """
        if authenticated:
            query_parameters['api_key'] = self._key

        url += "/" + self._lang.name
        for var in url_variables:
            url += "/" + var

        result = None
        if len(query_parameters):
            result = requests.get(url, params=query_parameters)
        else:
            result = requests.get(url)

        result.raise_for_status()
        return result

    def _getDate(self, daystr):
        """
        Accepts a string of the form "YYYY-MM-DD" and returns a date object representing that date, if daystr is the
        empty string, None is returned.

        :param daystr: A string in the format "YYYY-MM-DD" representing a date
        :return: A date object determined by the input string
        """
        if daystr == "":
            return None
        else:
            return datetime.date(int(daystr[0:4]), int(daystr[5:7]), int(daystr[8:10]))


from enum import Enum

class LegislationType(Enum):
    Regulation = 0
    Statute = 1


class Jurisdiction(Enum):
    ca = 0
    ab = 1
    mb = 2
    nb = 3
    nl = 4
    ns = 5
    nt = 6
    nu = 7
    on = 8
    pe = 9
    qc = 10
    sk = 11
    yk = 12
    bc = 13


class Language(Enum):
    fr = 0
    en = 1  # because english is greater than french


class DateScheme(Enum):
    ENTRY_INTO_FORCE = 0
    DOWNLOAD_DATE = 1



from setuptools import setup

setup(
    name = "pycanlii",
    description = "A python wrapper for the CanLII API",
    packages = ["pycanlii"],
    version = "1.0.3b1",
    author = "Jonathan Webb",
    home_page = "http://sherlocke.me/developers/",
    install_requires=["requests"]
    )


import pycanlii.pycanliibase as base
import pycanlii.enums as enums
import requests

class CaseDatabase(base.PyCanliiBase):
    """
    A database of CanLII Cases. This object is both indexable and iterable. These are both abstractions of the API.
    The API allows a maximum of 10000 cases at a time, so when populating the database it will grab all of the cases
    for you. If you are doing a loop or indexing very far into a CaseDatabase that is very large, be wary of 403 response codes,
    as by default you're limited to 10 requests per second by the CanLII API. When a 403 response code is received an
    HTTPError will be raised.

    Attributes:
        :name: A string representing the name of this CaseDatabase
        :id: A string representing the ID of this caseDatabase
        :jurisdiction: A Jurisdiction enum instance representing the Jurisdiction of this case
    """

    def __init__(self, data, apikey, language=enums.Language.en):
        base.PyCanliiBase.__init__(self, apikey, language)
        self.name = data['name']
        self.id = data["databaseId"]
        self.jurisdiction = enums.Jurisdiction[data['jurisdiction']]
        self._cases = []
        self._index = 0
        self._full = False

    def _getCases(self, extension=10000):
        cases = self._request("http://api.canlii.org/v1/caseBrowse", True, self.id,
                              offset=self._index, resultCount=extension).json()['cases']

        self._index += len(cases)
        if (len(cases) < extension):
            self._full = True

        for case in cases:
            self._cases.append(Case(case, self._key))


    def __iter__(self):
        i = 0
        while(not self._full):
            self._getCases()
            while i < self._index:
                yield self._cases[i]
                i += 1

    def __getitem__(self, item):
        while(self._index <= item):
            self._getCases(extension=item+1)
        return self._cases[item]



class Case(base.PyCanliiBase):
    """
    An object representing a CanLII Case

    Attributes:
        :content: A BeautifulSoup object representing the HTML content of this case
        :caseId: A string representing the caseId of this case
        :databaseId: A string representing the databaseId of this case
        :title: A string representing the title of this case
        :citation: A string representing the citation of this case
        :url: A string representing the URL where this case can be found
        :docketNumber: A string representing the docketNumber of this case
        :startDate: A date object  date of this case
    """

    def __init__(self, data, apikey):
        caseid = data["caseId"]
        if type(caseid) == str:
            self.caseId = caseid
        elif 'en' in caseid:
            self.caseId = caseid["en"]
            language = enums.Language.en
        else:
            self.caseId = caseid["fr"]
            language = enums.Language.fr
        base.PyCanliiBase.__init__(self, apikey, language)
        self.databaseId = data['databaseId']
        self.title = data['title']
        self.citation = data['citation']

        self._populated = False
        self._url = None
        self._docketNumber = None
        self._decisionDate = None

        # Used to store the content of the case
        self._content = None

    def _populate(self):
        if not self._populated:
            case = self._request("http://api.canlii.org/v1/caseBrowse", True, self.databaseId, self.caseId)
            case = case.json()
            self._url = case['url']
            self._docketNumber = case['docketNumber']
            self._decisionDate = self._getDate(case['decisionDate'])

            self._populated = True

    @property
    def content(self):
        if not self._content:
            req = requests.get(self.url)
            self._content = req.content

        return self._content

    def citedCases(self):
        """
        Returns a list of up to a maximum of 5 cases that are cited by this one.

        :return: A list of up to a maximum of 5 cases that are cited by this one, if there are no cited cases, None is returned
        """
        response = self._request("http://api.canlii.org/v1/caseCitatorTease/", True,
                            self.databaseId, self.caseId, "citedCases").json()

        if "citedCases" in response:
            cases = response["citedCases"]
            l = []
            for case in cases:
                l.append(Case(case, self._key))
            return l
        else:
            return None

    def citingCases(self):
        """
        Returns a list of up to a maximum of 5 cases that are citing this one.

        :return: A list of up to a maximum of 5 cases that are citing this one, if there are no citing cases, None is returned
        """
        response = self._request("http://api.canlii.org/v1/caseCitatorTease/", True,
                            self.databaseId, self.caseId, "citingCases").json()

        if "citingCases" in response:
            cases = response["citingCases"]
            l = []
            for case in cases:
                l.append(Case(case, self._key))
            return l
        else:
            return None

    def citedLegislation(self):
        """
        Returns a list of up to a maximum of 5 pieces of legislation that are cited by this one.

        :return: A list of up to a maximum of 5 pieces of legislation that are cited by this one, if there are no cited
         legislation, None is returned
        """
        response = self._request("http://api.canlii.org/v1/caseCitatorTease/", True,
                            self.databaseId, self.caseId, "citedLegislations").json()

        if "citedCases" in response:
            cases = response["citedLegislation"]
            l = []
            for case in cases:
                l.append(Case(case, self._key))
            return l
        else:
            return None

    @property
    def url(self):
        self._populate()
        return self._url

    @property
    def docketNumber(self):
        self._populate()
        return self._docketNumber

    @property
    def decisionDate(self):
        self._populate()
        return self._decisionDate
